/**************************************************************************************************
  Filename:       rsa_basic.c
  Revised:        $Date: 2012-11-21 09:13:07 -0800 (Wed, 21 Nov 2012) $
  Revision:       $Revision: 32272 $

  Description:    RemoTI Sample Application (RSA) for CC2530 Remote Control.

  Copyright 2008-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/


/**************************************************************************************************
 *                                           Includes
 */

/* Hal Driver includes */
#include "hal_adc.h"
#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
#include "hal_buzzer.h"
#endif
#include "hal_types.h"
#include "hal_key.h"
#include "hal_timer.h"
#include "hal_drivers.h"
#include "hal_led.h"
#include "hal_lcd.h"

#if (defined FEATURE_ZID) && (FEATURE_ZID == TRUE)
#include "zid_common.h"
#include "zid_class_device.h"
#include "zid_hid_constants.h"
#include "zid_profile.h"
#endif //(defined FEATURE_ZID) && (FEATURE_ZID == TRUE)

/* OS includes */
#include "OSAL.h"
#include "OSAL_Tasks.h"
#include "OSAL_PwrMgr.h"

/* Application Includes */
#include "OnBoard.h"

/* RemoTI API */
#include "rti_constants.h"
#include "rti.h"

/* Application */
#include "rsa.h"

#ifdef FEATURE_OAD
#include "oad_client.h"
#include "oad.h"
#endif // FEATURE_OAD


#ifdef HAL_IRGEN 
#include "hal_irgen.h"
#include "hal_irgen_NEC.h"
#endif

/**************************************************************************************************
 *                                           Macros
 */

/**************************************************************************************************
 *                                           Constant
 */

// RSA Events
#define RSA_EVT_INIT                    0x0001
#define RSA_EVT_REBOOT                  0x0002
#define RSA_EVT_OAD_INACTIVITY          0x0004
#define RSA_EVT_RANDOM_BACKOFF_TIMER    0x0008
#ifdef HAL_IRGEN
#define RSA_IRG_EVT_IR_START            0x0010  // Event to start IR generation
#define RSA_IRG_EVT_IR_REPEAT_TIMER     0x0020  // Event for IR repeat timer (114 ms)
#define RSA_IRG_EVT_IR_RELEASED_TIMER   0x0040  // Event for CERC button released detection timer
#define RSA_IRG_EVT_IR_COMPLETED        0x0080  // Event to indicates IR generation has completed
#endif

// OAD polling timeout duration in 200 ms
#define RSA_POLL_TIMEOUT                 200

// RSA States
enum
{
  RSA_STATE_INIT,
  RSA_STATE_READY,
  RSA_STATE_PAIR,
  RSA_STATE_NDATA,
  RSA_STATE_UNPAIR,
  RSA_STATE_TEST,
  RSA_STATE_TEST_COMPLETED,
  RSA_STATE_OAD
};

// -- media selections: these are arbitrary --
// these are secondary codes based on an initial key press that corresponds to the Media Function
#define RSA_MEDIA_RECORDED_TV  1
#define RSA_MEDIA_LIVE_TV      2
#define RSA_MEDIA_END          3

// -- special key map command codes --

// media selection
#define RSA_MEDIA_SEL            0x80

// vendor specific extended TV command set range for keymap
#define RSA_EXT_TV               (RSA_MEDIA_SEL + RSA_MEDIA_END)

// Note that sequence has to be maintained to use function pointer array
#define RSA_ACT_START            0xB0
#define RSA_ACT_PAIR             0xB0
#define RSA_ACT_UNPAIR           0xB1
#define RSA_ACT_TOGGLE_TGT       0xB2
#define RSA_ACT_TEST_MODE        0xB3
#ifdef FEATURE_OAD
#define RSA_ACT_POLL             0xB4
#endif

// target device type selection key
#define RSA_TGT_TYPE             0xC0

// key mapping table
typedef struct
{
  uint8 key;
  uint8 profile;
} rsaKeyMap_t;

// Supported target device types: maximum up to 6 device types.
static const uint8 tgtList[RTI_MAX_NUM_SUPPORTED_TGT_TYPES] =
{
  RTI_DEVICE_TELEVISION,
  RTI_DEVICE_VIDEO_PLAYER_RECORDER,
  RTI_DEVICE_SET_TOP_BOX,
  RTI_DEVICE_MEDIA_CENTER_PC,
  RTI_DEVICE_RESERVED_INVALID,
  RTI_DEVICE_RESERVED_INVALID
};

// List of implemented device types: maximum up to 3 device types.
static const uint8 devList[RTI_MAX_NUM_DEV_TYPES] =
{
  RTI_DEVICE_REMOTE_CONTROL,
  RTI_DEVICE_RESERVED_INVALID,
  RTI_DEVICE_RESERVED_INVALID
};

// List of implemented profiles: maximum up to 7 profiles.
static const uint8 profileList[RTI_MAX_NUM_PROFILE_IDS] =
{
  RTI_PROFILE_ZRC, 0, 0, 0, 0, 0, 0
};

static const uint8 vendorName[RTI_VENDOR_STRING_LENGTH] = "TI-LPRF";

#if (!defined FEATURE_ZID) || (FEATURE_ZID == FALSE)
// CC2530 Remote keymap
static const rsaKeyMap_t __code rsaKeyMap[48] =
{
  // 0b00 <KPb> <KPa>
  // row mapped    to P0 and P1
  // column mapped to shift register controlled by P0 and P2
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 0000
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 0001
  { RTI_CERC_ROOT_MENU, RTI_PROFILE_ZRC },                   // 0b00 00 0010   - MENU
  { RTI_CERC_LEFT, RTI_PROFILE_ZRC },                        // 0b00 00 0011   - NAV LEFT
  { RTI_CERC_REWIND, RTI_PROFILE_ZRC },                      // 0b00 00 0100   - REV
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 0101   - MOUSE LEFT
  { RTI_CERC_VOLUME_DOWN, RTI_PROFILE_ZRC },                 // 0b00 00 0110   - VOL-
  { RTI_CERC_VOLUME_UP, RTI_PROFILE_ZRC },                   // 0b00 00 0111   - VOL+
  { RTI_CERC_RECORD, RTI_PROFILE_ZRC },                      // 0b00 00 1000   - REC
  { RTI_CERC_NUM_1, RTI_PROFILE_ZRC },                       // 0b00 00 1001   - 1
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 1010   - AV
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 1011
  { RSA_ACT_PAIR, RTI_PROFILE_RTI },                         // 0b00 00 1100   - RED (K1)
  { RTI_CERC_NUM_7, RTI_PROFILE_ZRC },                       // 0b00 00 1101   - 7
  { RTI_CERC_NUM_4, RTI_PROFILE_ZRC },                       // 0b00 00 1110   - 4
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 00 1111
  { RTI_CERC_NEXT_FAVORITE, RTI_PROFILE_ZRC },               // 0b00 01 0000   - FAV
  { RTI_CERC_VIDEO_ON_DEMAND, RTI_PROFILE_ZRC },             // 0b00 01 0001   - VOD
  { RTI_CERC_DOWN, RTI_PROFILE_ZRC },                        // 0b00 01 0010   - NAV DOWN
  { RTI_CERC_SELECT, RTI_PROFILE_ZRC },                      // 0b00 01 0011   - OK
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 01 0100
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 01 0101   - MOUSE MIDDLE
  { RTI_CERC_UP, RTI_PROFILE_ZRC },                          // 0b00 01 0110   - NAV UP
  { RTI_CERC_MUTE, RTI_PROFILE_ZRC },                        // 0b00 01 0111   - MUTE
  { RTI_CERC_PLAY, RTI_PROFILE_ZRC },                        // 0b00 01 1000   - PLAY
  { RTI_CERC_NUM_2, RTI_PROFILE_ZRC },                       // 0b00 01 1001   - 2
  { RTI_CERC_NUM_0, RTI_PROFILE_ZRC },                       // 0b00 01 1010   - 0
#ifdef FEATURE_OAD
  { RSA_ACT_POLL, RTI_PROFILE_RTI },                         // 0b00 01 1011   - YELLOW (K3)
#else
  { RTI_CERC_F4_YELLOW, RTI_PROFILE_ZRC },                   // 0b00 01 1011   - YELLOW (K3)
#endif
  { RSA_ACT_TEST_MODE, RTI_PROFILE_RTI },                    // 0b00 01 1100   - GREEN (K2)
  { RTI_CERC_NUM_8, RTI_PROFILE_ZRC },                       // 0b00 01 1101   - 8
  { RTI_CERC_NUM_5, RTI_PROFILE_ZRC },                       // 0b00 01 1110   - 5
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 01 1111

  { RTI_CERC_POWER_TOGGLE_FUNCTION, RTI_PROFILE_ZRC },       // 0b00 10 0000   - POWER
  { RSA_ACT_TOGGLE_TGT, RTI_PROFILE_RTI },                   // 0b00 10 0001   - TV
  { RTI_CERC_BACKWARD, RTI_PROFILE_ZRC },                    // 0b00 10 0010   - BACK
  { RTI_CERC_RIGHT, RTI_PROFILE_ZRC },                       // 0b00 10 0011   - NAV RIGHT
  { RTI_CERC_FAST_FORWARD, RTI_PROFILE_ZRC },                // 0b00 10 0100   - FFWD
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 10 0101   - MOUSE RIGHT
  { RTI_CERC_CHANNEL_DOWN, RTI_PROFILE_ZRC },                // 0b00 10 0110   - CH-
  { RTI_CERC_CHANNEL_UP, RTI_PROFILE_ZRC },                  // 0b00 10 0111   - CH+
  { RTI_CERC_STOP, RTI_PROFILE_ZRC },                        // 0b00 10 1000   - STOP
  { RTI_CERC_NUM_3, RTI_PROFILE_ZRC },                       // 0b00 10 1001   - 3
  { RTI_CERC_PAUSE, RTI_PROFILE_ZRC },                       // 0b00 10 1010   - PAUSE (?)
  { RTI_CERC_F1_BLUE, RTI_PROFILE_ZRC },                     // 0b00 10 1011   - BLUE (K4)
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 10 1100
  { RTI_CERC_NUM_9, RTI_PROFILE_ZRC },                       // 0b00 10 1101   - 9
  { RTI_CERC_NUM_6, RTI_PROFILE_ZRC },                       // 0b00 10 1110   - 6
  { RTI_CERC_RESERVED_1, RTI_PROFILE_ZRC },                  // 0b00 10 1111
};
#elif (defined FEATURE_ZID) && (FEATURE_ZID == TRUE)
// CC2530 Remote keymap
static const rsaKeyMap_t __code rsaKeyMap[48] =
{
  // 0b00 <KPb> <KPa>
  // row mapped    to P0 and P1
  // column mapped to shift register controlled by P0 and P2
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 00 0000
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 00 0001
  { RTI_CERC_ROOT_MENU, RTI_PROFILE_ZRC },                   // 0b00 00 0010   - MENU
  { HID_KEYBOARD_LEFT_ARROW, RTI_PROFILE_ZID },              // 0b00 00 0011   - NAV LEFT
  { RTI_CERC_REWIND, RTI_PROFILE_ZRC },                      // 0b00 00 0100   - REV
  { HID_MOUSE_BUTTON_LEFT, RTI_PROFILE_ZID },                // 0b00 00 0101   - MOUSE LEFT
  { RTI_CERC_VOLUME_DOWN, RTI_PROFILE_ZRC },                 // 0b00 00 0110   - VOL-
  { RTI_CERC_VOLUME_UP, RTI_PROFILE_ZRC },                   // 0b00 00 0111   - VOL+
  { RTI_CERC_RECORD, RTI_PROFILE_ZRC },                      // 0b00 00 1000   - REC
  { HID_KEYBOARD_1, RTI_PROFILE_ZID },                       // 0b00 00 1001   - 1
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 00 1010   - AV
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 00 1011
  { RSA_ACT_PAIR, RTI_PROFILE_RTI },                         // 0b00 00 1100   - RED (K1)
  { HID_KEYBOARD_7, RTI_PROFILE_ZID },                       // 0b00 00 1101   - 7
  { HID_KEYBOARD_4, RTI_PROFILE_ZID },                       // 0b00 00 1110   - 4
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 00 1111
  { RTI_CERC_NEXT_FAVORITE, RTI_PROFILE_ZRC },               // 0b00 01 0000   - FAV
  { RTI_CERC_VIDEO_ON_DEMAND, RTI_PROFILE_ZRC },             // 0b00 01 0001   - VOD
  { HID_KEYBOARD_DOWN_ARROW, RTI_PROFILE_ZID },              // 0b00 01 0010   - NAV DOWN
  { HID_KEYBOARD_RETURN, RTI_PROFILE_ZID },                  // 0b00 01 0011   - OK
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 01 0100
  { HID_MOUSE_BUTTON_MIDDLE, RTI_PROFILE_ZID },              // 0b00 01 0101   - MOUSE MIDDLE
  { HID_KEYBOARD_UP_ARROW, RTI_PROFILE_ZID },                // 0b00 01 0110   - NAV UP
  { RTI_CERC_MUTE, RTI_PROFILE_ZRC },                        // 0b00 01 0111   - MUTE
  { RTI_CERC_PLAY, RTI_PROFILE_ZRC },                        // 0b00 01 1000   - PLAY
  { HID_KEYBOARD_2, RTI_PROFILE_ZID },                       // 0b00 01 1001   - 2
  { HID_KEYBOARD_0, RTI_PROFILE_ZID },                       // 0b00 01 1010   - 0
#ifdef FEATURE_OAD
  { RSA_ACT_POLL, RTI_PROFILE_RTI },                         // 0b00 01 1011   - YELLOW (K3)
#else
  { RTI_CERC_F4_YELLOW, RTI_PROFILE_ZRC },                   // 0b00 01 1011   - YELLOW (K3)
#endif
  { RSA_ACT_TEST_MODE, RTI_PROFILE_RTI },                    // 0b00 01 1100   - GREEN (K2)
  { HID_KEYBOARD_8, RTI_PROFILE_ZID },                       // 0b00 01 1101   - 8
  { HID_KEYBOARD_5, RTI_PROFILE_ZID },                       // 0b00 01 1110   - 5
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 01 1111

  { RTI_CERC_POWER_TOGGLE_FUNCTION, RTI_PROFILE_ZRC },       // 0b00 10 0000   - POWER
  { RSA_ACT_TOGGLE_TGT, RTI_PROFILE_RTI },                   // 0b00 10 0001   - TV
  { RTI_CERC_BACKWARD, RTI_PROFILE_ZRC },                    // 0b00 10 0010   - BACK
  { HID_KEYBOARD_RIGHT_ARROW, RTI_PROFILE_ZID },             // 0b00 10 0011   - NAV RIGHT
  { RTI_CERC_FAST_FORWARD, RTI_PROFILE_ZRC },                // 0b00 10 0100   - FFWD
  { HID_MOUSE_BUTTON_RIGHT, RTI_PROFILE_ZID },               // 0b00 10 0101   - MOUSE RIGHT
  { RTI_CERC_CHANNEL_DOWN, RTI_PROFILE_ZRC },                // 0b00 10 0110   - CH-
  { RTI_CERC_CHANNEL_UP, RTI_PROFILE_ZRC },                  // 0b00 10 0111   - CH+
  { RTI_CERC_STOP, RTI_PROFILE_ZRC },                        // 0b00 10 1000   - STOP
  { HID_KEYBOARD_3, RTI_PROFILE_ZID },                       // 0b00 10 1001   - 3
  { RTI_CERC_PAUSE, RTI_PROFILE_ZRC },                       // 0b00 10 1010   - PAUSE (?)
  { RTI_CERC_F1_BLUE, RTI_PROFILE_ZRC },                     // 0b00 10 1011   - BLUE (K4)
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID },                // 0b00 10 1100
  { HID_KEYBOARD_9, RTI_PROFILE_ZID },                       // 0b00 10 1101   - 9
  { HID_KEYBOARD_6, RTI_PROFILE_ZID },                       // 0b00 10 1110   - 6
  { HID_KEYBOARD_RESERVED, RTI_PROFILE_ZID }                 // 0b00 10 1111
};
#endif

// IR signal toggle bit
#define RSA_IR_TOGGLE_BIT           1

/* Define to TRUE to enable sending a control released command frame even when a
 * control repeated command frame was not sent.
 */
#if !defined RSA_KEY_RELEASED_VERBOSE
#define RSA_KEY_RELEASED_VERBOSE    FALSE
#endif

/**************************************************************************************************
 *                                          Typedefs
 */

// Test request command
// Test parameters structure should be compliant with the TI vendor specific command format.
typedef struct
{
  uint8 startCondition;
  uint8 testType;
  uint8 txOptions;
  uint8 userDataSize; // user data is one byte plus this size as one byte should be used
                      // to carray test command identifier
  uint16 numPackets;
  uint16 maxBackoffDuration;
} rrtTestReq_t;

enum
{
  RSA_TEST_TERMINATE,
  RSA_TEST_LATENCY,
  RSA_TEST_PER
};

enum
{
  RSA_TEST_START_IMMEDIATE,
  RSA_TEST_START_ON_BUTTON
};

typedef struct
{
  uint8 testType;
  struct
  {
    uint16 numPackets;
    uint16 bin[29];
  } latency;
} rrtTestReportRsp_t;

/**************************************************************************************************
 *                                        Global Variables
 */

uint8 RSA_TaskId;

/**************************************************************************************************
 *                                        Local Variables
 */

// current target device pairing reference
static uint8 rsaDestIndex;

// current state
static uint8 rsaState;

// key state
static uint8 rsaKeyRepeated;

// designated target device type
static uint8 rsaTgtDevType;

// Restricted to length RTI_NWK_USER_STRING_LENGTH-1 (one byte for null character)
//static uint8 userString[] = "TI RF4CE LPWSW";

static rrtTestReq_t rsaTestReqCache =
{
  RSA_TEST_START_IMMEDIATE,
  RSA_TEST_LATENCY,
  RTI_TX_OPTION_ACKNOWLEDGED | RTI_TX_OPTION_VENDOR_SPECIFIC,
  8,
  1000,
  64
};

static rrtTestReportRsp_t rsaTestReportCache;

// Max app payload, used for vendor specific data
static uint8 pTestData[128];

// Measurement variables
static uint32 rsaSendReqTimeStamp;
static uint32 rsaSendCnfTimeStamp;

struct {
  // count of successful confirmations within 100ms, 200ms, 300ms, ...
  uint16 latencyCnt[11];
  uint16 failCnt;
  uint16 reqCnt;
} rsaAutoTestStatistics;

#ifdef HAL_IRGEN
// Command for IR signal generation
static uint32 irgCmd;

// Current IR signal generation state
static uint8 rsaIrgenState;

// IrGenApp IR generation states
enum
{
  RSA_IRG_STATE_IRGEN_READY = 0,  // Ready for immediate IR signal generation
  RSA_IRG_STATE_IRGEN_REPEAT      // Repeat the previous command
};
#endif

static __no_init rcnNwkPairingEntry_t rsaPairingEntryBuf;
static uint8 rsaMaxNumPairingEntries;

// Function prototypes for forward reference follow
static void rsaConfig(void);
static void rsaSelectInitialTargetDevice(void);
static void rsaPairKeyAction(void);
static void rsaUnpairKeyAction(void);
static void rsaTargetDeviceToggleKeyAction(void);
static void rsaToggleTestModeKeyAction(void);
#ifdef FEATURE_OAD
static void rsaPollKeyAction(void);
#endif
static void rsaRRTRunTest(uint8);
static void rsaRRTSendData(void);
static void rsaRRTSendReport(void);

// function pointer map for special key actions
typedef void (*rsaActionFn_t)(void);

static const rsaActionFn_t rsaSpecialKeyActions[] =
{
  rsaPairKeyAction,                // set state to discovery, and turn on LED2
  rsaUnpairKeyAction,
  rsaTargetDeviceToggleKeyAction,
  rsaToggleTestModeKeyAction
#ifdef FEATURE_OAD
    , rsaPollKeyAction
#endif
};

/**************************************************************************************************
 *
 * @fn          RSA_Init
 *
 * @brief       Initialize the application
 *
 * @param       taskId - taskId of the task after it was added in the OSAL task queue
 *
 * @return      none
 */
void RSA_Init(uint8 taskId)
{
  // initialize the task id
  RSA_TaskId = taskId;

  // set sample application state to not ready until RTI Init is successfully confirmed
  rsaState = RSA_STATE_INIT;

  // schedule Remote Application startup
  osal_set_event( RSA_TaskId, RSA_EVT_INIT );

#ifdef CC2430RC
  // Special code for CC2430RC to turn off LEDs in order not to drain current
  if (HAL_LED == FALSE)
  {
    P1_0 = 0;
    P1_1 = 0;
  }
#endif

#ifdef HAL_IRGEN  
  // Initialize IR generation driver
  HalIrGenInitNec();
#endif
  
  rsaTgtDevType = RTI_DEVICE_RESERVED_FOR_WILDCARDS;
}

/**************************************************************************************************
 *
 * @fn          RSA_ProcessEvent
 *
 * @brief       This routine handles events
 *
 * @param       taskId - ID of the application task when it registered with the OSAL
 *              events - Events for this task
 *
 * @return      16bit - Unprocessed events
 */
uint16 RSA_ProcessEvent(uint8 taskId, uint16 events)
{
  uint8* pMsg;

  if (events & SYS_EVENT_MSG)
  {
    while ((pMsg = osal_msg_receive(taskId)) != NULL)
    {
      (void)osal_msg_deallocate((uint8 *) pMsg);
    }

    return events ^ SYS_EVENT_MSG;
  }

  // Basic Remote Application Initialization
  if ( events & RSA_EVT_INIT )
  {
    if ( rsaState == RSA_STATE_INIT )
    {
      rsaConfig();  // Configure RTI parameters relevant to the RSA.

      // Initialize the RF4CE software and begin network operation.
      // NOTE: Configuration Parameters need to be set prior to this call.
      RTI_InitReq();

      // Enable power savings.
      // This function can be called irrespective of RTI_InitReq() status.
      RTI_EnableSleepReq();
    }
  }

  // The following are latency test related codes.
  if (events & RSA_EVT_RANDOM_BACKOFF_TIMER)
  {
    if (rsaState == RSA_STATE_TEST)
    {
      uint8 i, profileId, vendorId;

      rsaAutoTestStatistics.reqCnt++;
      rsaTestReportCache.latency.numPackets++;


      // send the data over the air
      profileId = RTI_PROFILE_ZRC;
      vendorId  = RTI_VENDOR_TEXAS_INSTRUMENTS;
      pTestData[0] = RTI_PROTOCOL_TEST;
      pTestData[1] = RTI_CMD_TEST_DATA;
      for (i = 0; i < rsaTestReqCache.userDataSize; i++)
      {
        pTestData[i+2] = i;
      }

      rsaSendReqTimeStamp = osal_GetSystemClock();
      RTI_SendDataReq( rsaDestIndex, profileId, vendorId, rsaTestReqCache.txOptions, rsaTestReqCache.userDataSize+2, pTestData);
    }
  }

#ifdef FEATURE_OAD
  if ( events & RSA_EVT_REBOOT )
  {
    OAD_Reboot();
  }
  else if (( events & RSA_EVT_OAD_INACTIVITY ) && rsaState == RSA_STATE_OAD )
  {
    // OAD inactivity timer expired
    rsaState = RSA_STATE_READY;
    OAD_State = OAD_CLIENT_IDLE_STATE;
  }
#endif
  
#ifdef HAL_IRGEN  
  // Start IR signal generation events
  if (events & RSA_IRG_EVT_IR_START)
  {
    /* Tell OSAL to hold power while transmitting IR */
    osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_HOLD );
    
    // Start released detection timer
    osal_start_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_RELEASED_TIMER, 100);
    
    // Generate IR signal
    HalIrGenCommandNec(irgCmd);
    
    // repeat the command till key release is detected or released timer expires
    rsaIrgenState = RSA_IRG_STATE_IRGEN_REPEAT;
    
    // Start IR signal generation repeat timer
    osal_start_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_REPEAT_TIMER, HAL_IRGEN_NEC_REPEAT_INTERVAL);
  }
  
  // Repeat timer has expired, send repeat frame
  if (events & RSA_IRG_EVT_IR_REPEAT_TIMER)
  {
    // Make sure we are in repeat state
    if (rsaIrgenState == RSA_IRG_STATE_IRGEN_REPEAT)
    {      
      // Generate repeat signal
      HalIrGenRepeatNec();
      
      // restart repeat timer
      osal_start_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_REPEAT_TIMER, HAL_IRGEN_NEC_REPEAT_INTERVAL);
    }
    else // This condition is not supposed to happen but added for completeness
    {
      rsaIrgenState = RSA_IRG_STATE_IRGEN_READY;
    }   
  }
  
  // Release detection timer has expired.  This means the ZRC release frame
  // was not received - we need to stop the IR repeat frame signal generation
  if (events & RSA_IRG_EVT_IR_RELEASED_TIMER) 
  {  
    // stop repetition
    if (rsaIrgenState == RSA_IRG_STATE_IRGEN_REPEAT)
    {
      osal_stop_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_REPEAT_TIMER);
      rsaIrgenState = RSA_IRG_STATE_IRGEN_READY;
    }
  }
  
  // IR signal generation completed
  if (events & RSA_IRG_EVT_IR_COMPLETED) 
  {  
    // Biip to signal end of IR
#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
    /* Tell OSAL to not go to sleep because buzzer uses T3 */
    osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_HOLD );

    HalBuzzerRing( 50, RSA_BuzzerCompleteCback ); // ring buzzer for 500msec
#else    
    /* Tell OSAL it's OK to go to sleep */
    osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_CONSERVE );
#endif
  }
#endif

  return 0;
}

/**************************************************************************************************
 *
 * @fn      RTI_InitCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_InitReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_InitReq has returned.
 *
 * @param   status - Result of RTI_InitReq API call.
 *
 * @return  void
 */
void RTI_InitCnf( rStatus_t status )
{
  if ( status == RTI_SUCCESS )
  {
    RTI_ReadItemEx(RTI_PROFILE_RTI, RTI_CONST_ITEM_MAX_PAIRING_TABLE_ENTRIES, 1,
                                                       &rsaMaxNumPairingEntries);

    // RTI has been successfully started, so application is now ready to go
    rsaState = RSA_STATE_READY;

    // select initial target device type
    // currently, this routine finds the first pairing entry index, if one exists
    rsaSelectInitialTargetDevice();

    // startup control settings
    {
      uint8 pValue = RESTORE_STATE;
      RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_STARTUP_CTRL, 1, &pValue);
    }

    /* Provide feedback that initialization was successful */
#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
    /* Tell OSAL to not go to sleep because buzzer uses T3 */
    osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_HOLD );

    HalBuzzerRing( 250, RSA_BuzzerCompleteCback ); // ring buzzer for 500msec
#endif
  }
}

/**************************************************************************************************
 *
 * @fn      RTI_PairCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_PairReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_PairReq has returned.
 *
 * @param   status - Result of RTI_PairReq API call.
 * @param   dstIndex - Pairing table index of paired device, or invalid.
 * @param   devType  - Pairing table index device type, or invalid.
 * @return  void
 */
void RTI_PairCnf( rStatus_t status, uint8 dstIndex, uint8 devType )
{
  (void) devType; // unused argument

#if (defined HAL_LED) && (HAL_LED == TRUE)
  // turn off transmission light indicator
  HalLedSet(HAL_LED_2, HAL_LED_MODE_OFF);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)

  /* Provide feedback that pairing was successful */
#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
  /* Tell OSAL to not go to sleep because buzzer uses T3 */
  osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_HOLD );

  HalBuzzerRing( 500, RSA_BuzzerCompleteCback ); // ring buzzer for 500msec
#endif

  if ( status == RTI_SUCCESS )
  {
    rsaDestIndex = dstIndex;
  }

  rsaState = RSA_STATE_READY;
}

/**************************************************************************************************
 *
 * @fn      RTI_PairAbortCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_PairAbortReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_PairAbortReq has returned.
 *
 * @param   status - Result of RTI_PairAbortReq API call.
 * @return  void
 */
void RTI_PairAbortCnf( rStatus_t status )
{
  (void) status; // unused argument

  if (RSA_STATE_PAIR == rsaState)
  {
#if (defined HAL_LED) && (HAL_LED == TRUE)
    // turn off transmission light indicator
    HalLedSet(HAL_LED_2, HAL_LED_MODE_OFF);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)

    rsaState = RSA_STATE_READY;
  }
}

/**************************************************************************************************
 *
 * @fn      RTI_AllowPairCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_AllowPairReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_AllowPairReq has returned.
 * @param   status   - Result of RTI_PairReq API call.
 * @param   dstIndex - Pairing table entry of paired device, or invalid
 * @param   devType  - Pairing table index device type, or invalid
 *
 * @return  void
 */
void RTI_AllowPairCnf( rStatus_t status, uint8 dstIndex, uint8 devType )
{
  // Do nothing.
  // Controller does not trigger AllowPairReq() and hence is not expecting this callback.

  // unused arguments
  (void) status;
  (void) dstIndex;
  (void) devType;
}

/**************************************************************************************************
 *
 * @fn      RTI_UnpairCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_UnpairReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_UnpairReq has returned.
 *
 * @param   status   - Result of RTI_PairReq API call.
 * @param   dstIndex - Pairing table index of paired device, or invalid.
 *
 * @return  void
 */
void RTI_UnpairCnf( rStatus_t status, uint8 dstIndex )
{
  // unused arguments
  (void) status;
  (void) dstIndex;
}

/**************************************************************************************************
 *
 * @fn      RTI_UnpairInd
 *
 * @brief   RTI indication callback initiated by receiving unpair request command.
 *
 * @param   dstIndex - Pairing table index of paired device.
 *
 * @return  void
 */
void RTI_UnpairInd( uint8 dstIndex )
{
  // unused arguments
  (void) dstIndex;
}

/**************************************************************************************************
 *
 * @fn      RTI_SendDataCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_SendDataReq API
 *          call. The client is expected to complete this function.
 *
 *          NOTE: It is possible that this call can be made to the RTI client
 *                before the call to RTI_SendDataReq has returned.
 *
 * @param   status - Result of RTI_SendDataReq API call.
 *
 * @return  void
 */
void RTI_SendDataCnf( rStatus_t status )
{
  if ( status == RTI_SUCCESS )
  {
    if (rsaState == RSA_STATE_NDATA)
    {
#if (defined HAL_LED) && (HAL_LED == TRUE)
      HalLedSet(HAL_LED_1, HAL_LED_MODE_OFF);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      rsaState = RSA_STATE_READY;
    }
    else if (rsaState == RSA_STATE_TEST)
    {
      uint32 time;

      // Record latecy
      rsaSendCnfTimeStamp = osal_GetSystemClock();
      time = rsaSendCnfTimeStamp - rsaSendReqTimeStamp;
      if (time < 1000)
      {
        if (time < 100)
        {
          rsaTestReportCache.latency.bin[time/10]++;
        }
        else
        {
          rsaTestReportCache.latency.bin[(time-100)/50 + 10]++;
        }
      }
      else
      {
        rsaTestReportCache.latency.bin[28]++;
      }

      if (rsaTestReportCache.latency.numPackets ==
          rsaTestReqCache.numPackets)
      {
        rsaState = RSA_STATE_TEST_COMPLETED;
#if (defined HAL_LED) && (HAL_LED == TRUE)
        // TURN ON LED
        HalLedSet(HAL_LED_1, HAL_LED_MODE_ON);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      }
      else
      {
        rsaRRTSendData();
      }
    }
    else if (rsaState == RSA_STATE_TEST_COMPLETED)
    {
      rsaState = RSA_STATE_TEST;
#if (defined HAL_LED) && (HAL_LED == TRUE)
      HalLedBlink(HAL_LED_1, 0, 50, 2000);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
    }
  }
  else
  {
    if (rsaState == RSA_STATE_NDATA)
    {
#if (defined HAL_LED) && (HAL_LED == TRUE)
      HalLedSet(HAL_LED_1, HAL_LED_MODE_OFF);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      rsaState = RSA_STATE_READY;
    }
    else if (rsaState == RSA_STATE_TEST)
    {
      rsaTestReportCache.latency.bin[28]++;

      if (rsaTestReportCache.latency.numPackets ==
          rsaTestReqCache.numPackets)
      {
        rsaState = RSA_STATE_TEST_COMPLETED;
#if (defined HAL_LED) && (HAL_LED == TRUE)
        // TURN ON LED
        HalLedSet(HAL_LED_1, HAL_LED_MODE_ON);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      }
      else
      {
        rsaRRTSendData();
      }
    }
  }
}

/**************************************************************************************************
 *
 * @fn      RTI_RxEnableCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_RxEnableReq API
 *          call. The client is expected to complete this function.
 *
 * @param   status - Result of RTI_EnableRxReqReq API call.
 *
 * @return  void
 */
void RTI_RxEnableCnf( rStatus_t status )
{
  // Do nothing

  (void) status; // unused argument
}

/**************************************************************************************************
 *
 * @fn      RTI_EnableSleepCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_EnableSleepReq API
 *          call. The client is expected to complete this function.
 *
 * @param   status - Result of RTI_EnableSleepReq API call.
 *
 * @return  void
 *
 */
void RTI_EnableSleepCnf( rStatus_t status )
{
  // Do nothing

  (void) status; // unused argument
}

/**************************************************************************************************
 *
 * @fn      RTI_DisableSleepCnf
 *
 * @brief   RTI confirmation callback initiated by client's RTI_DisableSleepReq API
 *          call. The client is expected to complete this function.
 *
 * @param   status - Result of RTI_EnableSleepReq API call.
 *
 * @return  void
 *
 */
void RTI_DisableSleepCnf( rStatus_t status )
{
  // Do nothing

  (void) status; // unused argument
}

/**************************************************************************************************
 *
 * @fn      RTI_ReceiveDataInd
 *
 * @brief   RTI receive data indication callback asynchronous initiated by
 *          another node. The client is expected to complete this function.
 *
 * input parameters
 *
 * @param   srcIndex:  Pairing table index.
 * @param   profileId: Profile identifier.
 * @param   vendorId:  Vendor identifier.
 * @param   rxLQI:     Link Quality Indication.
 * @param   rxFlags:   Receive flags.
 * @param   len:       Number of bytes to send.
 * @param   *pData:    Pointer to data to be sent.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 *
 */
void RTI_ReceiveDataInd( uint8 srcIndex, uint8 profileId, uint16 vendorId, uint8 rxLQI, uint8 rxFlags, uint8 len, uint8 *pData )
{
  (void) srcIndex; // unused argument
  (void) profileId; // unused argument
  (void) rxLQI; // unused argument

  if (len && (vendorId == RTI_VENDOR_TEXAS_INSTRUMENTS) && (rxFlags & RTI_RX_FLAGS_VENDOR_SPECIFIC))
  {
    if (pData[0] == RTI_PROTOCOL_TEST && rsaState == RSA_STATE_TEST)
    {
      if (pData[1] == RTI_CMD_TEST_PARAMETERS)
      {
        osal_memcpy(&rsaTestReqCache, &pData[2], (sizeof(rrtTestReq_t)));
      }
    }
#ifdef FEATURE_OAD
    else if (pData[0] == RTI_PROTOCOL_OAD &&
             (rsaState == RSA_STATE_READY || rsaState == RSA_STATE_OAD))
    {
      // Profile identifier is not used for TI vendor specific commands
      // as proper use of profile identifier is questionable.
      OAD_ReceiveDataInd(srcIndex, len, pData);
      if (OAD_State == OAD_CLIENT_IDLE_STATE)
      {
        // Set the state back to non-OAD state
        rsaState = RSA_STATE_READY;
        // Stop OAD inactivity watchdog timer
        osal_stop_timerEx(RSA_TaskId, RSA_EVT_OAD_INACTIVITY);
      }
      else
      {
        rsaState = RSA_STATE_OAD;
        if (OAD_State == OAD_CLIENT_REBOOT)
        {
          // Delay a bit till last OAD message is passed to server and reboot
          osal_start_timerEx(RSA_TaskId, RSA_EVT_REBOOT, 1000);
        }
        else
        {
          // (re-)start inactivity watchdog timer
          // This shall be application specific timer but has to be bigger
          // than longest inactivity of OAD protocol, which is usually the time
          // it takes for OAD client to calculate CRC and send back enable confirm.
          osal_start_timerEx(RSA_TaskId, RSA_EVT_OAD_INACTIVITY, 5000);
        }
      }
    }
#endif
  }
}

/**************************************************************************************************
 *
 * @fn      RSA_KeyCback
 *
 * @brief   Callback service for keys
 *
 * @param   keys  - key that was pressed (i.e. the scanned row/col index)
 *          state - shifted
 *
 * @return  void
 */
void RSA_KeyCback(uint8 keys, uint8 state)
{
#if !RSA_KEY_RELEASED_VERBOSE
  static uint8 rsaRepeatCommandIssued;  // repeat command was issued last time
#endif
  static uint8 rsaLastKey = HAL_KEY_CODE_NOKEY;
  uint8 cmd = rsaKeyMap[keys].key; // gets the key code based on the row/col index
  uint8 profile = rsaKeyMap[keys].profile;

  (void) state; // unused argument

  if (keys == HAL_KEY_CODE_NOKEY)
  {
    cmd = RTI_CERC_RESERVED_1;  // Overwrite the invalid cmd lookup on key release.
    rsaKeyRepeated = 0;
  }
  else if (cmd == RTI_CERC_RESERVED_1)
  {
    // no command defined for key, so no more to do
    return;
  }
  else if (rsaLastKey == keys)
  {
    if (rsaKeyRepeated < 255)
    {
      rsaKeyRepeated++;
    }
  }
  else
  {
    rsaLastKey = keys;  // remember last row/col index
    rsaKeyRepeated = 0;
#if !RSA_KEY_RELEASED_VERBOSE
    rsaRepeatCommandIssued = FALSE;
#endif
  }

  // Check if we are in test mode since keys will have differnt mapping
  if (rsaState == RSA_STATE_TEST && !rsaKeyRepeated && keys != HAL_KEY_CODE_NOKEY)
  {
    // Check if toogle key is pressed and go back to normal
    if ( cmd == RSA_ACT_TEST_MODE )
    {
      // Toggle test mode
      rsaSpecialKeyActions[3]();
      return;
    }

    //perform test
    rsaRRTRunTest(cmd);
    return;
  }

  if (rsaState == RSA_STATE_TEST_COMPLETED && !rsaKeyRepeated)
  {
    if (cmd == RTI_CERC_SELECT)
    {
      rsaRRTSendReport();
      return;
    }
  }

  if (RSA_STATE_PAIR == rsaState && RSA_ACT_PAIR == cmd && 0 == rsaKeyRepeated)
  {
    // Pairing already in progress
    // Pair key is used to toggle pairing action
    RTI_PairAbortReq();
    return;
  }

  if ((rsaState != RSA_STATE_READY) && (rsaState != RSA_STATE_INIT))
  {
    if (keys == HAL_KEY_CODE_NOKEY)
    {
      rsaLastKey = HAL_KEY_CODE_NOKEY;
#if !RSA_KEY_RELEASED_VERBOSE
      rsaRepeatCommandIssued = FALSE;
#endif
    }

    // Keys are handled only in ready state
    return;
  }

  // some non-CERC keys are used to perform RTI tasks, so check that here
  if (cmd >= RSA_TGT_TYPE && cmd != RTI_CERC_RESERVED_1)
  {
    // Select a target device type
    rsaTgtDevType = cmd - RSA_TGT_TYPE;
    // Toggle key action will switch to the proper target device
    // in addtional it would toggle between targets with the same matching device type.
    rsaTargetDeviceToggleKeyAction();

    // Reset target device type so that generic device toggle key would work
    // with any device types.
    rsaTgtDevType = RTI_DEVICE_RESERVED_FOR_WILDCARDS;
  }
  else if (cmd >= RSA_ACT_START && cmd != RTI_CERC_RESERVED_1)
  {
    // action table command
    rsaSpecialKeyActions[cmd - RSA_ACT_START]();
  }
  // else we have a valid CERC key command or TI extended TV command,
  // so as long as we are ready, and currently paired with a target, process the key command
  else if ( rsaState == RSA_STATE_READY )
  {
#ifdef HAL_IRGEN
    // First prepare IR, then move on to RF
    // button pressed
    if ( (0 == rsaKeyRepeated) && (keys != HAL_KEY_CODE_NOKEY) )
    {
      switch (cmd) 
      {   
      case RTI_CERC_PLAY: // Play button
        irgCmd = HAL_IRGEN_CMD_NEC(0xAB, 0x23);
        break;
      case RTI_CERC_PAUSE_PLAY_FUNCTION: // Play/Pause button
        irgCmd = HAL_IRGEN_CMD_NEC(0xAB, 0x23);
        break;
        
      default:
        return;        
      }
      
      // Set event to start IR Signal generation
      osal_set_event(RSA_TaskId, RSA_IRG_EVT_IR_START);
    }
    else if (keys == HAL_KEY_CODE_NOKEY)
    {
      // stop release detection and repeat timer
      osal_stop_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_RELEASED_TIMER);
      osal_stop_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_REPEAT_TIMER);
    }
    else
    {
      // restart released detection timer
      osal_start_timerEx(RSA_TaskId, RSA_IRG_EVT_IR_RELEASED_TIMER, 100);
    }
#endif //HAL_IRGEN
    
    if (rsaDestIndex != RTI_INVALID_PAIRING_REF)
    {
      uint8 len = 1;
      uint8 pData[5];  // Note that array size has to match maximum possible data length
      uint8 txOptions = RTI_TX_OPTION_ACKNOWLEDGED;
      uint8 profileId = profile;
      uint16 vendorId = RTI_VENDOR_TEXAS_INSTRUMENTS;
      
      // ready to send a CERC command
      len = 1;
      
      // first check if the key command is power toggle, and handle as a special case
      if ( cmd == RTI_CERC_POWER_TOGGLE_FUNCTION || cmd == RTI_CERC_POWER)
      {
        // power toggle key should not repeat and hence as toggle key is
        // repeatedly pressed, it should be ignored.
        if ( rsaKeyRepeated )
        {
          return;
        }
      }
      // If STOP key is pressed for 5 seconds, clear the pairing table
      if ( cmd == RTI_CERC_STOP && rsaKeyRepeated >= 100)
      {
        // 5 seconds upon pressing stop key, clear pairing table
        uint8 value8 = CLEAR_STATE;
        (void) RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_STARTUP_CTRL, 1, &value8);
        
#ifdef FEATURE_OAD
        OAD_Reboot();
#else
        HAL_DISABLE_INTERRUPTS();
        RTI_SwResetReq();
        while (1)
        {
          asm("NOP");
        }
#endif // FEATURE_OAD
      }
      
      if (keys == HAL_KEY_CODE_NOKEY)
      {
#if !RSA_KEY_RELEASED_VERBOSE
        if (rsaRepeatCommandIssued)
        {
#endif
          pData[0] = RTI_CERC_USER_CONTROL_RELEASED;
          cmd = rsaKeyMap[rsaLastKey].key;  // Gets the last key code based on the row/col index.
          profileId = rsaKeyMap[rsaLastKey].profile;
          rsaLastKey = HAL_KEY_CODE_NOKEY;
#if !RSA_KEY_RELEASED_VERBOSE
          rsaRepeatCommandIssued = FALSE;
        }
        else
        {
          rsaLastKey = HAL_KEY_CODE_NOKEY;
          // key release does not translate into any key released command
          return;
        }
#endif
      }
      else if (rsaKeyRepeated)
      {
        pData[0] = RTI_CERC_USER_CONTROL_REPEATED;
#if !RSA_KEY_RELEASED_VERBOSE
        rsaRepeatCommandIssued = TRUE;
#endif
      }
      else if (cmd >= RSA_EXT_TV)  // TI vendor specific command for TV extension
      {
        pData[0] = RTI_PROTOCOL_EXT_TV_RC;
      }
      else  // CERC control command.
      {
        pData[0] = RTI_CERC_USER_CONTROL_PRESSED;
      }
      
      if (pData[0] == RTI_PROTOCOL_EXT_TV_RC)
      {
        pData[1] = cmd - RSA_EXT_TV;
        len = 2;
        txOptions |= RTI_TX_OPTION_VENDOR_SPECIFIC;
      }
      else if ((pData[0] == RTI_CERC_USER_CONTROL_PRESSED) ||
               (cmd < RSA_EXT_TV))  // CERC command repeat/release.
      {
        len++; // at least one more byte for RC command code
        
        // check if this key command is mapped to mutiple col/row indexes,
        //   which indicates multiple CERC subfunctions
        if ( cmd >= RSA_MEDIA_SEL )
        {
          pData[1] = RTI_CERC_SELECT_MEDIA_FUNCTION;
          
          // the select media function requires a media number, so increase
          //   length by one to handle the payload
          len++;
          
          // now fill in any extra payload
          pData[2] = cmd - RSA_MEDIA_SEL;
        }
        else
        {
          // set up plain RC command code
          pData[1] = cmd;
        }
      }
      
      // sending data state
      rsaState = RSA_STATE_NDATA;
      
#if (defined HAL_LED) && (HAL_LED == TRUE)
      // TURN ON LED
      HalLedSet(HAL_LED_1, HAL_LED_MODE_ON);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      
      // send the data over the air
      RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY_INDEX, 1, &rsaDestIndex);
#if RCN_FEATURE_SECURITY
      if ((RTI_ReadItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY, sizeof(rsaPairingEntryBuf),
                          (uint8 *) &rsaPairingEntryBuf) == RTI_SUCCESS) &&
          (rsaPairingEntryBuf.pairingRef != RTI_INVALID_PAIRING_REF) &&
            rsaPairingEntryBuf.securityKeyValid)
      {
        txOptions |= RTI_TX_OPTION_SECURITY;
      }
#endif
      RTI_SendDataReq( rsaDestIndex, profileId, vendorId, txOptions, len, pData);
    }
#ifdef HAL_IRGEN
    else
    {
      if (keys == HAL_KEY_CODE_NOKEY)
      {
        rsaLastKey = HAL_KEY_CODE_NOKEY;
      }
    }
#endif
  }
}

#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
/**************************************************************************************************
 *
 * @fn      RSA_BuzzerCompleteCback
 *
 * @brief   Callback function called when ringing of buzzer is done
 *
 * @param   none
 *
 * @return  void
 */
void RSA_BuzzerCompleteCback( void )
{
#if (defined HAL_BUZZER) && (HAL_BUZZER == TRUE)
  /* Tell OSAL it's OK to go to sleep */
  osal_pwrmgr_task_state( RSA_TaskId, PWRMGR_CONSERVE );
#endif
}
#endif //(defined HAL_BUZZER) && (HAL_BUZZER == TRUE)

/**************************************************************************************************
 *
 * @fn      rsaConfig
 *
 * @brief   Configure RTI parameters specific to the ZRC Remote Controller.
 *
 * @param
 *
 * None.
 *
 * @return  None.
 *
 */
static void rsaConfig(void)
{
  union { // Space for largest number of bytes, not counting strings.
    uint8 TgtTypes[RTI_MAX_NUM_SUPPORTED_TGT_TYPES];
    uint8 DevList[RTI_MAX_NUM_DEV_TYPES];
    uint8 VendorName[RTI_VENDOR_STRING_LENGTH];
    uint8 ProfileList[RTI_MAX_NUM_PROFILE_IDS];
    uint8 Buf[SADDR_EXT_LEN];
  } u;

  (void)osal_memcpy(u.TgtTypes, tgtList, RTI_MAX_NUM_SUPPORTED_TGT_TYPES);
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_NODE_SUPPORTED_TGT_TYPES,
                                         RTI_MAX_NUM_SUPPORTED_TGT_TYPES, u.TgtTypes);

  // No User String pairing; 1 Device (Remote Control); 1 Profile (ZRC)
  u.Buf[0] = RTI_BUILD_APP_CAPABILITIES(0, 1, 1);
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_APPL_CAPABILITIES, 1, u.Buf);

  (void)osal_memcpy(u.DevList, devList,  RTI_MAX_NUM_DEV_TYPES);
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_APPL_DEV_TYPE_LIST,
                                         RTI_MAX_NUM_DEV_TYPES, u.DevList);

  (void)osal_memcpy(u.ProfileList, profileList, RTI_MAX_NUM_PROFILE_IDS);
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_APPL_PROFILE_ID_LIST,
                                                RTI_MAX_NUM_PROFILE_IDS, u.ProfileList);

  // Controller Type; No A/C Pwr; Security capable; Channel Normalization capable.
  u.Buf[0] = RTI_BUILD_NODE_CAPABILITIES(0, 0, 1, 1);
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_NODE_CAPABILITIES, 1, u.Buf);

  *(uint16 *)u.Buf = RTI_VENDOR_TEXAS_INSTRUMENTS;
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_VENDOR_ID, 2, u.Buf);

  (void)osal_memcpy(u.VendorName, vendorName, sizeof(vendorName));
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_CP_ITEM_VENDOR_NAME,
                                              sizeof(vendorName), u.VendorName);
}

/**************************************************************************************************
 * @fn          rsaSelectInitialTargetDevice
 *
 * @brief       This routine is for the case where the RC boots, and there already are
 *              pairing table entries, so which target does the RC pair with? This routine chooses
 *              the last valid pairing entry.
 * Note: this logic of selection may not be very useful when the pairing table size is huge.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaSelectInitialTargetDevice(void)
{
  uint8 maxIdx = rsaMaxNumPairingEntries;
  rsaDestIndex = RTI_INVALID_PAIRING_REF;

  for (uint8 i = 0; i < maxIdx; i++)
  {
    (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY_INDEX, 1, &i);

    // RTI_ReadItemEx succeeds only when pairing entry pointed by current entry index is valid.
    if (RTI_ReadItemEx(RTI_PROFILE_RTI,  RTI_SA_ITEM_PT_CURRENT_ENTRY,
            sizeof(rsaPairingEntryBuf), (uint8 *)&rsaPairingEntryBuf) == RTI_SUCCESS)
    {
      rsaDestIndex = i;  // Latch the last valid entry.
    }
  }

  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY_INDEX, 1, &rsaDestIndex);
}

/**************************************************************************************************
 * @fn          rsaPairKeyAction
 *
 * @brief       This function performs action required when a pair trigger key is depressed.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaPairKeyAction(void)
{
  if (0 == rsaKeyRepeated)
  {
    // pair key only acts upon key press.
    // Allowing key depress to repeat pairing would conflict with pair toggling
    rsaState = RSA_STATE_PAIR;

#if (defined HAL_LED) && (HAL_LED == TRUE)
    // IR generation uses the same pin for LED_2, hence do not use LED_2
    HalLedSet(HAL_LED_2, HAL_LED_MODE_ON);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)

    // begin pairing
    RTI_PairReq();
  }
}

/**************************************************************************************************
 * @fn          rsaUnpairKeyAction
 *
 * @brief       This function performs action required when un-pair trigger key is depressed.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaUnpairKeyAction(void)
{
  rsaState = RSA_STATE_UNPAIR;
}

/**************************************************************************************************
 * @fn          rsaTargetDeviceToggleKeyAction
 *
 * @brief       This function performs action required when target device toggle key is depressed.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaTargetDeviceToggleKeyAction(void)
{
  uint8 maxCnt = rsaMaxNumPairingEntries - 1;
  uint8 maxIdx = rsaMaxNumPairingEntries;
  uint8 idx = (rsaDestIndex == RTI_INVALID_PAIRING_REF) ? 0 : rsaDestIndex;

  if (rsaKeyRepeated)  // Toggle-Key should not repeat; force user to release key and press again.
  {
    return;
  }

  for (uint8 cnt = 0; cnt < maxCnt; cnt++)
  {
    if (++idx == maxIdx)
    {
      idx = 0;
    }

    // Set current pairing table item to the current search index in order to read it.
    (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY_INDEX, 1, &idx);

    // RTI_ReadItemEx succeeds only when pairing entry pointed by current entry index is valid.
    if (RTI_ReadItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY,
                sizeof(rsaPairingEntryBuf), (uint8 *)&rsaPairingEntryBuf) == RTI_SUCCESS)
    {
      if (rsaTgtDevType == RTI_DEVICE_RESERVED_FOR_WILDCARDS)
      {
        rsaDestIndex = idx;
        return;
      }

      for (uint8 type = 0; type < RCN_MAX_NUM_DEV_TYPES; type++)
      {
        if (rsaPairingEntryBuf.devTypeList[type] == rsaTgtDevType)
        {
          rsaDestIndex = idx;
          return;
        }
      }
    }
  }

  // No valid entry to toggle to, so restore the originally selected entry.
  (void)RTI_WriteItemEx(RTI_PROFILE_RTI, RTI_SA_ITEM_PT_CURRENT_ENTRY_INDEX, 1, &rsaDestIndex);
}

#ifdef FEATURE_OAD
/**************************************************************************************************
 * @fn          rsaPollKeyAction
 *
 * @brief       This function performs action required when poll key is depressed.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaPollKeyAction(void)
{
  uint8 data = RTI_PROTOCOL_POLL;

  if (rsaKeyRepeated == 0 && rsaState == RSA_STATE_READY && rsaDestIndex != RTI_INVALID_PAIRING_REF)
  {
    // Turn on receiver for some duration
    RTI_RxEnableReq(RSA_POLL_TIMEOUT);

    // Send poll message
    // Note that if poll message were to be triggered periodically, it is better
    // to use unacknowledged transmission for power saving purpose.
    RTI_SendDataReq(rsaDestIndex,
                    OAD_PROFILE_ID,
                    OAD_VENDOR_ID,
                    OAD_TX_OPTIONS,
                    1,
                    &data);

    rsaState = RSA_STATE_NDATA;
  }
}
#endif // FEATURE_OAD


#ifdef HAL_IRGEN
/******************************************************************************
 * @fn      HalIrGenIsrCback
 *
 * @brief   This function is a callback function that a user application has to
 *          define. HAL IR signal generation driver will call this callback
 *          function upon completion of IR signal generation.
 *
 *          NOTE that this function is called in the ISR context and should as
 *          such do a minimum of processsing. A prudent design will set an
 *          application event to delay further processing
*
* input parameters
*
* None
*
* output parameters
*
* None.
*
* @return  None.
*
*/
void HalIrGenIsrCback(void)
{
  // Set event to avoid processing in isr
  osal_set_event(RSA_TaskId, RSA_IRG_EVT_IR_COMPLETED);
}
#endif

/**************************************************************************************************
 * @fn          rsaToggleTestModeKeyAction
 *
 * @brief       This function performs action required when toggle test mode key is depressed.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaToggleTestModeKeyAction(void)
{
  if (!rsaKeyRepeated)
  {
    if (rsaState != RSA_STATE_TEST)
    {
      // Turn on RX in case target send test configuration
      RTI_RxEnableReq(RTI_RX_ENABLE_ON);

#if (defined HAL_LED) && (HAL_LED == TRUE)
      // slow blink LED to indicate ready for test
      HalLedBlink(HAL_LED_1, 0, 50, 2000);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      rsaState = RSA_STATE_TEST;

    }
    else if (rsaState == RSA_STATE_TEST)
    {
      // Turn off RX, going back to normal mode
      RTI_RxEnableReq(RTI_RX_ENABLE_OFF);

#if (defined HAL_LED) && (HAL_LED == TRUE)
      // turn off LED indicating normal mode
      HalLedSet(HAL_LED_1, HAL_LED_MODE_OFF);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
      rsaState = RSA_STATE_READY;
    }
  }
}

/**************************************************************************************************
 * @fn          rsaRRTRunTest
 *
 * @brief       This function initiates automated testing
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaRRTRunTest(uint8 cmd)
{
  //Reset stats in case test is run again
  osal_memset(&rsaTestReportCache, 0, sizeof(rsaTestReportCache));

  // latency test
  if (cmd == RTI_CERC_NUM_1)
  {
    rsaTestReportCache.testType = RSA_TEST_LATENCY;
#if (defined HAL_LED) && (HAL_LED == TRUE)
    HalLedBlink(HAL_LED_1, 0, 50, 500);
#endif //(defined HAL_LED) && (HAL_LED == TRUE)
    rsaRRTSendData();
  }
}

/**************************************************************************************************
 * @fn          rsaRRTSendData
 *
 * @brief       This function sends a test data
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
static void rsaRRTSendData(void)
{
   if (rsaTestReqCache.maxBackoffDuration > 0)
   {
    // Not a good random number generation mechanism
    // but this will do as a sample application
    uint16 seed = osal_rand();

    osal_start_timerEx(RSA_TaskId,
                       RSA_EVT_RANDOM_BACKOFF_TIMER,
                       (seed % rsaTestReqCache.maxBackoffDuration) + 1);

   }
   else
   {
     osal_set_event(RSA_TaskId, RSA_EVT_RANDOM_BACKOFF_TIMER);
   }
}

static void rsaRRTSendReport(void)
{
  uint8 pData[sizeof(rsaTestReportCache)+2];
  uint8 profileId, vendorId;

  profileId = RTI_PROFILE_ZRC;
  vendorId  = RTI_VENDOR_TEXAS_INSTRUMENTS;
  pData[0] = RTI_PROTOCOL_TEST;
  pData[1] = RTI_CMD_TEST_REPORT;

  osal_memcpy(&pData[2], &rsaTestReportCache, sizeof(rsaTestReportCache));

  RTI_SendDataReq( rsaDestIndex, profileId, vendorId, rsaTestReqCache.txOptions, sizeof(pData), pData);
}

/**************************************************************************************************
*/
